@@ -13,23 +13,23 @@ from ooni.geoip import ProbeIP
 from ooni.settings import config
 
 from ooni.deckgen import __version__
-from ooni.resources import inputs
-
+from ooni.deckgen.processors import citizenlab_test_lists
+from ooni.deckgen.processors import namebench_dns_servers
+from ooni.resources.update import download_resources
 
 class Options(usage.Options):
     synopsis = """%s [options]
     """ % sys.argv[0]
 
     optParameters = [
-        ["country-code", "c",
-         None,
+        ["country-code", "c", None,
          "Specify the two letter country code for which we should "
-         "generate the deck."
-         ],
-        ["output", "o",
-         None,
-         "Specify the directory where to write output."
-         ]
+         "generate the deck."],
+        ["collector", None, None, "Specify a custom collector to use when "
+                                  "submitting reports"],
+        ["bouncer", None, None, "Specify a custom bouncer to use"],
+        ["output", "o", None,
+         "Specify the directory where to write output."]
     ]
 
     def opt_version(self):
@@ -40,77 +40,70 @@ class Options(usage.Options):
 class Deck(object):
     _base_entry = {
         "options": {
+            "test_file": None,
+            "subargs": [],
+            "annotations": None,
+
             "collector": None,
-            "help": 0,
-            "logfile": None,
-            "no-default-reporter": 0,
-            "parallelism": None,
-            "pcapfile": None,
+            "bouncer": None,
+
             "reportfile": None,
-            "resume": 0,
-            "testdeck": None
+
+            "no-collector": 0,
+            "no-geoip": 0,
+            "no-yamloo": 0,
+            "verbose": 0
         }
     }
 
-    def __init__(self):
-        self.deck = []
+    def __init__(self, collector=None, bouncer=None):
+        self.deck_entries = []
+        self.collector = collector
+        self.bouncer = bouncer
 
     def add_test(self, test_file, subargs=[]):
         deck_entry = copy.deepcopy(self._base_entry)
+        deck_entry['options']['collector'] = self.collector
+        deck_entry['options']['bouncer'] = self.bouncer
         deck_entry['options']['test_file'] = test_file
         deck_entry['options']['subargs'] = subargs
-        self.deck.append(deck_entry)
+        self.deck_entries.append(deck_entry)
 
     def pprint(self):
-        print yaml.safe_dump(self.deck)
+        print yaml.safe_dump(self.deck_entries)
 
     def write_to_file(self, filename):
         with open(filename, "w+") as f:
-            f.write(yaml.safe_dump(self.deck))
+            f.write(yaml.safe_dump(self.deck_entries))
 
 
 def generate_deck(options):
-    dns_servers_processor = inputs['namebench-dns-servers.csv']['processor']
-    url_lists_processor = inputs['citizenlab-test-lists.zip']['processor']
-
+    url_list_country = None
     try:
-        url_list_country = url_lists_processor.generate_country_input(
+        url_list_country = citizenlab_test_lists.generate_country_input(
             options['country-code'],
             options['output']
         )
-
     except Exception:
         print "Could not generate country specific url list"
         print "We will just use the global one."
-        url_list_country = None
 
-    url_list_global = url_lists_processor.generate_global_input(
-        options['output']
-    )
-    dns_servers = dns_servers_processor.generate_country_input(
-        options['country-code'],
+    url_list_global = citizenlab_test_lists.generate_global_input(
         options['output']
     )
 
-    deck = Deck()
-    # deck.add_test('manipulation/http_host', ['-f', 'somefile.txt'])
-    deck.add_test('blocking/http_requests', ['-f', url_list_global])
-    deck.add_test('blocking/dns_consistency',
-                  ['-f', url_list_global, '-T', dns_servers])
-
-    if url_list_country is not None:
-        deck.add_test('blocking/dns_consistency',
-                      ['-f', url_list_country, '-T', dns_servers])
-        deck.add_test('blocking/http_requests', ['-f', url_list_country])
-
+    deck = Deck(collector=options['collector'], bouncer=options['bouncer'])
     deck.add_test('manipulation/http_invalid_request_line')
     deck.add_test('manipulation/http_header_field_manipulation')
-    # deck.add_test('manipulation/traceroute')
+
+    if url_list_country is not None:
+        deck.add_test('blocking/web_connectivity', ['-f', url_list_country])
+    deck.add_test('blocking/web_connectivity', ['-f', url_list_global])
+
     if config.advanced.debug:
         deck.pprint()
     deck_filename = os.path.join(options['output'],
-                                 "%s-%s-user.deck" % (__version__,
-                                                      options['country-code']))
+                                 "default-user.deck")
     deck.write_to_file(deck_filename)
     print "Deck written to %s" % deck_filename
     print "Run ooniprobe like so:"
@@ -124,6 +117,20 @@ def get_user_country_code():
     yield probe_ip.lookup()
     defer.returnValue(probe_ip.geodata['countrycode'])
 
+def resources_up_to_date():
+    if config.get_data_file_path("GeoIP/GeoIP.dat") is None:
+        return False
+
+    if config.get_data_file_path("resources/"
+                                 "namebench-dns-servers.csv") is None:
+        return False
+
+    if config.get_data_file_path("resources/"
+                                 "citizenlab-test-lists/"
+                                 "global.csv") is None:
+        return False
+
+    return True
 
 @defer.inlineCallbacks
 def run():
@@ -135,6 +142,11 @@ def run():
         print options
         sys.exit(1)
 
+    if not resources_up_to_date():
+        print("Resources for running ooniprobe are not up to date.")
+        print("Will update them now.")
+        yield download_resources()
+
     if not options['output']:
         options['output'] = os.getcwd()
 
