@@ -7,13 +7,11 @@ from ooni.utils import log, generate_filename
 from ooni.utils.net import randomFreePort
 from ooni.nettest import NetTest, getNetTestInformation
 from ooni.settings import config
-from ooni import errors
 from ooni.nettest import test_class_name_to_name
 
-from txtorcon import TorConfig, TorState, launch_tor, build_tor_connection
+from ooni.utils.onion import start_tor, connect_to_control_port
 
-from twisted.internet import defer, reactor
-from twisted.internet.endpoints import TCP4ClientEndpoint
+from twisted.internet import defer
 
 
 class Director(object):
@@ -61,7 +59,7 @@ class Director(object):
 
     _scheduledTests = 0
     # Only list NetTests belonging to these categories
-    categories = ['blocking', 'manipulation']
+    categories = ['blocking', 'manipulation', 'third_party']
 
     def __init__(self):
         self.activeNetTests = []
@@ -130,11 +128,10 @@ class Director(object):
         if start_tor:
             if check_incoherences:
                 yield config.check_tor()
-            if config.advanced.start_tor:
+            if config.advanced.start_tor and config.tor_state is None:
                 yield self.startTor()
-            elif config.tor.control_port:
-                log.msg("Connecting to Tor Control Port...")
-                yield self.getTorState()
+            elif config.tor.control_port and config.tor_state is None:
+                yield connect_to_control_port()
 
         if config.global_options['no-geoip']:
             aux = [False]
@@ -233,7 +230,7 @@ class Director(object):
 
     @defer.inlineCallbacks
     def startNetTest(self, net_test_loader, report_filename,
-                     collector_address=None):
+                     collector_client=None, no_yamloo=False):
         """
         Create the Report for the NetTest and start the report NetTest.
 
@@ -241,21 +238,24 @@ class Director(object):
             net_test_loader:
                 an instance of :class:ooni.nettest.NetTestLoader
         """
+        test_details = net_test_loader.getTestDetails()
+        test_cases = net_test_loader.getTestCases()
+
         if self.allTestsDone.called:
             self.allTestsDone = defer.Deferred()
 
-        if config.privacy.includepcap:
-            self.startSniffing(net_test_loader.testDetails)
+        if config.privacy.includepcap or config.global_options.get('pcapfile', None):
+            self.startSniffing(test_details)
+        report = Report(test_details, report_filename,
+                        self.reportEntryManager,
+                        collector_client,
+                        no_yamloo)
 
-        report = Report(net_test_loader.testDetails, report_filename,
-                        self.reportEntryManager, collector_address)
-
-        net_test = NetTest(net_test_loader, report)
+        yield report.open()
+        net_test = NetTest(test_cases, test_details, report)
         net_test.director = self
 
-        yield net_test.report.open()
-
-        yield net_test.initializeInputProcessor()
+        yield net_test.initialize()
         try:
             self.activeNetTests.append(net_test)
             self.measurementManager.schedule(net_test.generateMeasurements())
@@ -265,7 +265,7 @@ class Director(object):
         finally:
             self.netTestDone(net_test)
 
-    def startSniffing(self, testDetails):
+    def startSniffing(self, test_details):
         """ Start sniffing with Scapy. Exits if required privileges (root) are not
         available.
         """
@@ -274,12 +274,17 @@ class Director(object):
         if config.scapyFactory is None:
             config.scapyFactory = ScapyFactory(config.advanced.interface)
 
-        if not config.reports.pcap:
+        # XXX this is dumb option to have in the ooniprobe.conf. Drop it in
+        # the future.
+        prefix = config.reports.pcap
+        if prefix is None:
             prefix = 'report'
-        else:
-            prefix = config.reports.pcap
-        filename = config.global_options['reportfile'] if 'reportfile' in config.global_options.keys() else None
-        filename_pcap = generate_filename(testDetails, filename=filename, prefix=prefix, extension='pcap')
+
+        filename_pcap = config.global_options.get('pcapfile', None)
+        if filename_pcap is None:
+            filename_pcap = generate_filename(test_details,
+                                              prefix=prefix,
+                                              extension='pcap')
         if len(self.sniffers) > 0:
             pcap_filenames = set(sniffer.pcapwriter.filename for sniffer in self.sniffers.values())
             pcap_filenames.add(filename_pcap)
@@ -287,15 +292,10 @@ class Director(object):
                     ','.join(pcap_filenames))
 
         sniffer = ScapySniffer(filename_pcap)
-        self.sniffers[testDetails['test_name']] = sniffer
+        self.sniffers[test_details['test_name']] = sniffer
         config.scapyFactory.registerProtocol(sniffer)
         log.msg("Starting packet capture to: %s" % filename_pcap)
 
-    @defer.inlineCallbacks
-    def getTorState(self):
-        connection = TCP4ClientEndpoint(reactor, '127.0.0.1',
-                                        config.tor.control_port)
-        config.tor_state = yield build_tor_connection(connection)
 
     def startTor(self):
         """ Starts Tor
@@ -304,44 +304,16 @@ class Director(object):
         """
         log.msg("Starting Tor...")
 
-        @defer.inlineCallbacks
-        def state_complete(state):
-            config.tor_state = state
-            log.msg("Successfully bootstrapped Tor")
-            log.debug("We now have the following circuits: ")
-            for circuit in state.circuits.values():
-                log.debug(" * %s" % circuit)
-
-            socks_port = yield state.protocol.get_conf("SocksPort")
-            control_port = yield state.protocol.get_conf("ControlPort")
-
-            config.tor.socks_port = int(socks_port.values()[0])
-            config.tor.control_port = int(control_port.values()[0])
-
-        def setup_failed(failure):
-            log.exception(failure)
-            raise errors.UnableToStartTor
-
-        def setup_complete(proto):
-            """
-            Called when we read from stdout that Tor has reached 100%.
-            """
-            log.debug("Building a TorState")
-            config.tor.protocol = proto
-            state = TorState(proto.tor_protocol)
-            state.post_bootstrap.addCallback(state_complete)
-            state.post_bootstrap.addErrback(setup_failed)
-            return state.post_bootstrap
-
-        def updates(prog, tag, summary):
-            log.msg("%d%%: %s" % (prog, summary))
+        from txtorcon import TorConfig
 
         tor_config = TorConfig()
-        if config.tor.control_port:
-            tor_config.ControlPort = config.tor.control_port
+        if config.tor.control_port is None:
+            config.tor.control_port = int(randomFreePort())
+        if config.tor.socks_port is None:
+            config.tor.socks_port = int(randomFreePort())
 
-        if config.tor.socks_port:
-            tor_config.SocksPort = config.tor.socks_port
+        tor_config.ControlPort = config.tor.control_port
+        tor_config.SocksPort = config.tor.socks_port
 
         if config.tor.data_dir:
             data_dir = os.path.expanduser(config.tor.data_dir)
@@ -375,29 +347,7 @@ class Director(object):
         if os.geteuid() == 0:
             tor_config.User = pwd.getpwuid(os.geteuid()).pw_name
 
-        tor_config.save()
-
-        if not hasattr(tor_config, 'ControlPort'):
-            control_port = int(randomFreePort())
-            tor_config.ControlPort = control_port
-            config.tor.control_port = control_port
-
-        if not hasattr(tor_config, 'SocksPort'):
-            socks_port = int(randomFreePort())
-            tor_config.SocksPort = socks_port
-            config.tor.socks_port = socks_port
-
         tor_config.save()
         log.debug("Setting control port as %s" % tor_config.ControlPort)
         log.debug("Setting SOCKS port as %s" % tor_config.SocksPort)
-
-        if config.advanced.tor_binary:
-            d = launch_tor(tor_config, reactor,
-                           tor_binary=config.advanced.tor_binary,
-                           progress_updates=updates)
-        else:
-            d = launch_tor(tor_config, reactor,
-                           progress_updates=updates)
-        d.addCallback(setup_complete)
-        d.addErrback(setup_failed)
-        return d
+        return start_tor(tor_config)
