@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 import re
 import os
 import random
@@ -5,7 +6,6 @@ import random
 from hashlib import sha256
 
 from twisted.web import client, http_headers
-from ooni.utils.net import hasRawSocketPermission
 
 client._HTTP11ClientFactory.noisy = False
 
@@ -30,36 +30,26 @@ class GeoIPDataFilesNotFound(Exception):
 def IPToLocation(ipaddr):
     from ooni.settings import config
 
-    city_file = config.get_data_file_path('GeoIP/GeoLiteCity.dat')
     country_file = config.get_data_file_path('GeoIP/GeoIP.dat')
     asn_file = config.get_data_file_path('GeoIP/GeoIPASNum.dat')
 
     location = {'city': None, 'countrycode': 'ZZ', 'asn': 'AS0'}
+    if not asn_file or not country_file:
+        log.err("Could not find GeoIP data file in data directories."
+                "Try running ooniresources or"
+                " edit your ooniprobe.conf")
+        return location
 
-    def error():
-        log.err("Could not find GeoIP data file in %s."
-                "Try running ooniresources --update-geoip or"
-                " edit your ooniprobe.conf" % config.advanced.geoip_data_dir)
+    country_dat = GeoIP(country_file)
+    asn_dat = GeoIP(asn_file)
 
-    try:
-        country_dat = GeoIP(country_file)
-        location['countrycode'] = country_dat.country_code_by_addr(ipaddr)
-        if not location['countrycode']:
-            location['countrycode'] = 'ZZ'
-    except IOError:
-        error()
+    country_code = country_dat.country_code_by_addr(ipaddr)
+    if country_code is not None:
+        location['countrycode'] =  country_code
 
-    try:
-        city_dat = GeoIP(city_file)
-        location['city'] = city_dat.record_by_addr(ipaddr)['city']
-    except:
-        error()
-
-    try:
-        asn_dat = GeoIP(asn_file)
-        location['asn'] = asn_dat.org_by_addr(ipaddr).split(' ')[0]
-    except:
-        error()
+    asn = asn_dat.org_by_addr(ipaddr)
+    if asn is not None:
+        location['asn'] = asn.split(' ')[0]
 
     return location
 
@@ -74,10 +64,6 @@ def database_version():
         'GeoIPASNum': {
             'sha256': None,
             'timestamp': None
-        },
-        'GeoLiteCity': {
-            'sha256': None,
-            'timestamp': None
         }
     }
 
@@ -181,8 +167,6 @@ class ProbeIP(object):
         self.geodata['ip'] = self.address
         if not config.privacy.includeasn:
             self.geodata['asn'] = 'AS0'
-        if not config.privacy.includecity:
-            self.geodata['city'] = None
         if not config.privacy.includecountry:
             self.geodata['countrycode'] = 'ZZ'
         if not config.privacy.includeip:
@@ -190,34 +174,27 @@ class ProbeIP(object):
 
     @defer.inlineCallbacks
     def lookup(self):
-        try:
-            yield self.askTor()
-            log.msg("Found your IP via Tor %s" % self.address)
-            self.resolveGeodata()
+        if self.address:
             defer.returnValue(self.address)
-        except errors.TorStateNotFound:
-            log.debug("Tor is not running. Skipping IP lookup via Tor.")
-        except Exception:
-            log.msg("Unable to lookup the probe IP via Tor.")
+        else:
+            try:
+                yield self.askTor()
+                log.msg("Found your IP via Tor")
+                self.resolveGeodata()
+                defer.returnValue(self.address)
+            except errors.TorStateNotFound:
+                log.debug("Tor is not running. Skipping IP lookup via Tor.")
+            except Exception:
+                log.msg("Unable to lookup the probe IP via Tor.")
 
-        try:
-            yield self.askTraceroute()
-            log.msg("Found your IP via Traceroute %s" % self.address)
-            self.resolveGeodata()
-            defer.returnValue(self.address)
-        except errors.InsufficientPrivileges:
-            log.debug("Cannot determine the probe IP address with a traceroute, becase of insufficient privileges")
-        except:
-            log.msg("Unable to lookup the probe IP via traceroute")
-
-        try:
-            yield self.askGeoIPService()
-            log.msg("Found your IP via a GeoIP service: %s" % self.address)
-            self.resolveGeodata()
-            defer.returnValue(self.address)
-        except Exception:
-            log.msg("Unable to lookup the probe IP via GeoIPService")
-            raise
+            try:
+                yield self.askGeoIPService()
+                log.msg("Found your IP via a GeoIP service")
+                self.resolveGeodata()
+                defer.returnValue(self.address)
+            except Exception:
+                log.msg("Unable to lookup the probe IP via GeoIPService")
+                raise
 
     @defer.inlineCallbacks
     def askGeoIPService(self):
@@ -237,14 +214,6 @@ class ProbeIP(object):
         if not self.address:
             raise errors.ProbeIPUnknown
 
-    def askTraceroute(self):
-        """
-        Perform a UDP traceroute to determine the probes IP address.
-        """
-        if not hasRawSocketPermission():
-            raise errors.InsufficientPrivileges
-        raise NotImplemented
-
     def askTor(self):
         """
         Obtain the probes IP address by asking the Tor Control port via GET INFO
