@@ -11,12 +11,19 @@ from ooni.utils.net import userAgents
 from ooni.templates import httpt
 from ooni.errors import failureToString
 
+class MissingInput(Exception):
+    pass
 
 class UsageOptions(usage.Options):
     optParameters = [
         ['url', 'u', None, 'Specify a single URL to test.'],
         ['factor', 'f', 0.8,
-         'What factor should be used for triggering censorship (0.8 == 80%)']]
+         'What factor should be used for triggering censorship '
+         '(0.8 == 80%).']]
+    optFlags = [
+        ["withoutbody", "B", "don't include HTTP response body inside of the "
+         "report."],
+        ]
 
 
 class HTTPRequestsTest(httpt.HTTPTest):
@@ -30,10 +37,10 @@ class HTTPRequestsTest(httpt.HTTPTest):
     lengths match.
     """
     name = "HTTP Requests"
-    description = "Performs a HTTP GET request over Tor and one over the " \
-                  "local network and compares the two results."
+    description = ("Performs a HTTP GET request over Tor and one over the "
+                  "local network and compares the two results.")
     author = "Arturo Filastò"
-    version = "0.2.4"
+    version = "0.2.5"
 
     usageOptions = UsageOptions
 
@@ -47,6 +54,12 @@ class HTTPRequestsTest(httpt.HTTPTest):
     control_body_length = None
     experiment_body_length = None
 
+    def requirements(self):
+        if not self.localOptions['url'] and \
+                not self.localOptions['file']:
+            raise MissingInput("You did not specify either a URL with -u "
+                               "or an input file with -f")
+
     def setUp(self):
         """
         Check for inputs.
@@ -57,6 +70,7 @@ class HTTPRequestsTest(httpt.HTTPTest):
             self.url = self.localOptions['url']
         else:
             raise Exception("No input specified")
+        self.report['input'] = self.url
 
         self.factor = self.localOptions['factor']
         self.report['control_failure'] = None
@@ -66,6 +80,7 @@ class HTTPRequestsTest(httpt.HTTPTest):
         self.report['factor'] = float(self.factor)
         self.report['headers_diff'] = None
         self.report['headers_match'] = None
+        self.report['control_cloudflare'] = None
 
         self.headers = {'User-Agent': [random.choice(userAgents)]}
 
@@ -133,6 +148,11 @@ class HTTPRequestsTest(httpt.HTTPTest):
         if experiment and control:
             if hasattr(experiment, 'body') and hasattr(control, 'body') \
                     and experiment.body and control.body:
+                self.report['control_cloudflare'] = False
+                if 'Attention Required! | CloudFlare' in control.body:
+                    log.msg("The control body contains a blockpage from "
+                            "cloudflare. This will skew our results.")
+                    self.report['control_cloudflare'] = True
                 self.compare_body_lengths(len(control.body),
                                           len(experiment.body))
             if hasattr(experiment, 'headers') and hasattr(control, 'headers') \
