@@ -3,6 +3,7 @@ from __future__ import print_function
 import os
 import sys
 
+from ooni.constants import CANONICAL_BOUNCER_ONION
 from ooni.report import __version__
 from ooni.report import tool
 from ooni.settings import config
@@ -15,12 +16,18 @@ class Options(usage.Options):
     synopsis = """%s [options] upload | status
 """ % (os.path.basename(sys.argv[0]),)
 
+    optFlags = [
+        ["default-collector", "d", "Upload the reports to the default "
+                                   "collector that is looked up with the "
+                                   "canonical bouncer."]
+    ]
+
     optParameters = [
         ["configfile", "f", None,
          "Specify the configuration file to use."],
         ["collector", "c", None,
          "Specify the collector to upload the result to."],
-        ["bouncer", "b", "httpo://nkvphnp3p6agi5qq.onion",
+        ["bouncer", "b", None,
          "Specify the bouncer to query for a collector."]
     ]
 
@@ -53,10 +60,10 @@ def tor_check():
         sys.exit(1)
 
 
-def run():
+def run(args=sys.argv[1:]):
     options = Options()
     try:
-        options.parseOptions()
+        options.parseOptions(args)
     except Exception as exc:
         print("Error: %s" % exc)
         print(options)
@@ -64,6 +71,10 @@ def run():
     config.global_options = dict(options)
     config.set_paths()
     config.read_config_file()
+
+    if options['default-collector']:
+        options['bouncer'] = CANONICAL_BOUNCER_ONION
+
     if options['command'] == "upload" and options['report_file']:
         tor_check()
         return tool.upload(options['report_file'],
