@@ -1,3 +1,4 @@
+from base64 import b64encode
 from ooni.nettest import NetTestCase
 from ooni.utils import log
 from ooni.settings import config
@@ -6,6 +7,15 @@ from ooni.utils.net import hasRawSocketPermission
 from ooni.utils.txscapy import ScapySender, ScapyFactory
 
 
+def _representPacket(packet):
+    return {
+        "raw_packet": {
+            'data': b64encode(str(packet)),
+            'format': 'base64'
+        },
+        "summary": repr(packet)
+    }
+
 class BaseScapyTest(NetTestCase):
 
     """
@@ -88,8 +98,8 @@ class BaseScapyTest(NetTestCase):
                 sent_packet.src = '127.0.0.1'
                 received_packet.dst = '127.0.0.1'
 
-            self.report['sent_packets'].append(sent_packet)
-            self.report['answered_packets'].append(received_packet)
+            self.report['sent_packets'].append(_representPacket(sent_packet))
+            self.report['answered_packets'].append(_representPacket(received_packet))
         return packets
 
     def sr(self, packets, timeout=None, *arg, **kw):
@@ -142,7 +152,7 @@ class BaseScapyTest(NetTestCase):
 
         scapySender.stopSending()
         for sent_packet in packets:
-            self.report['sent_packets'].append(sent_packet)
+            self.report['sent_packets'].append(_representPacket(sent_packet))
 
 
 ScapyTest = BaseScapyTest
