@@ -8,6 +8,7 @@ from twisted.internet import defer
 from twisted.trial import unittest
 
 from ooni import errors as e
+from ooni.tests.mocks import MockCollectorClient
 from ooni.reporter import YAMLReporter, OONIBReporter, OONIBReportLog
 
 
@@ -25,14 +26,24 @@ test_details = {
     'software_version': '1.0',
     'input_hashes': [],
     'probe_asn': 'AS0',
-    'start_time': time.time()
+    'probe_cc': 'ZZ',
+    'test_start_time': '2016-01-01 22:33:11',
+    'data_format_version': '0.2.0'
 }
 
 oonib_new_report_message = {
-    'report_id': "2014-01-29T202038Z_AS0_" + "A" * 50,
+    'report_id': "20140129T202038Z_AS0_" + "A" * 50,
+    'backend_version': "1.0",
+    'supported_formats': ["yaml", "json"]
+}
+
+# This is used for testing legacy collectors
+oonib_new_report_yaml_message = {
+    'report_id': "20140129T202038Z_AS0_" + "A" * 50,
     'backend_version': "1.0"
 }
 
+
 oonib_generic_error_message = {
     'error': 'generic-error'
 }
@@ -48,7 +59,7 @@ class TestYAMLReporter(unittest.TestCase):
             os.remove(self.filename)
 
     def test_write_report(self):
-        y_reporter = YAMLReporter(test_details)
+        y_reporter = YAMLReporter(test_details, 'dummy-report.yaml')
         y_reporter.createReport()
         with open(y_reporter.report_path) as f:
             self.filename = y_reporter.report_path
@@ -62,42 +73,46 @@ class TestOONIBReporter(unittest.TestCase):
 
     def setUp(self):
         self.mock_response = {}
-        self.collector_address = 'http://example.com'
+
+        def mockRequest(method, urn, genReceiver, *args, **kw):
+            receiver = genReceiver(None, None)
+            return defer.maybeDeferred(receiver.body_processor,
+                                       json.dumps(self.mock_response))
+
+        mock_collector_client = MockCollectorClient('http://example.com')
+        mock_collector_client._request = mockRequest
 
         self.oonib_reporter = OONIBReporter(
             test_details,
-            self.collector_address)
-        self.oonib_reporter.agent = MagicMock()
-        self.mock_agent_response = MagicMock()
-
-        def deliverBody(body_receiver):
-            body_receiver.dataReceived(json.dumps(self.mock_response))
-            body_receiver.connectionLost(None)
-
-        self.mock_agent_response.deliverBody = deliverBody
-        self.oonib_reporter.agent.request.return_value = defer.succeed(
-            self.mock_agent_response)
+            mock_collector_client
+        )
 
     @defer.inlineCallbacks
     def test_create_report(self):
         self.mock_response = oonib_new_report_message
         yield self.oonib_reporter.createReport()
-        assert self.oonib_reporter.reportID == oonib_new_report_message[
-            'report_id']
+        self.assertEqual(self.oonib_reporter.reportId,
+                         oonib_new_report_message['report_id'])
 
     @defer.inlineCallbacks
     def test_create_report_failure(self):
         self.mock_response = oonib_generic_error_message
-        self.mock_agent_response.code = 406
         yield self.assertFailure(self.oonib_reporter.createReport(),
                                  e.OONIBReportCreationError)
 
     @defer.inlineCallbacks
     def test_write_report_entry(self):
+        self.mock_response = oonib_new_report_message
+        yield self.oonib_reporter.createReport()
         req = {'content': 'something'}
         yield self.oonib_reporter.writeReportEntry(req)
-        assert self.oonib_reporter.agent.request.called
 
+    @defer.inlineCallbacks
+    def test_write_report_entry_in_yaml(self):
+        self.mock_response = oonib_new_report_yaml_message
+        yield self.oonib_reporter.createReport()
+        req = {'content': 'something'}
+        yield self.oonib_reporter.writeReportEntry(req)
 
 class TestOONIBReportLog(unittest.TestCase):
 
